package org.openmuc.jdlms;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyInt;
import static org.powermock.api.mockito.PowerMockito.doAnswer;
import static org.powermock.api.mockito.PowerMockito.doCallRealMethod;
import static org.powermock.api.mockito.PowerMockito.doReturn;
import static org.powermock.api.mockito.PowerMockito.mock;
import static org.powermock.reflect.Whitebox.setInternalState;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.openmuc.jdlms.internal.APdu;
import org.openmuc.jdlms.internal.ReleaseResponseReason;
import org.openmuc.jdlms.internal.asn1.iso.acse.ACSE_apdu;
import org.openmuc.jdlms.internal.asn1.iso.acse.Association_information;
import org.openmuc.jdlms.internal.asn1.iso.acse.RLRE_apdu;
import org.openmuc.jdlms.internal.asn1.iso.acse.RLRQ_apdu;
import org.openmuc.jdlms.internal.asn1.iso.acse.Release_response_reason;
import org.openmuc.jdlms.sessionlayer.SessionLayer;
import org.openmuc.jdlms.settings.client.Settings;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@PrepareForTest(DlmsConnection.class)
@RunWith(PowerMockRunner.class)
public class GracefulDisonnectTest {

    @Test
    public void test1() throws Exception {
        DlmsConnection connection = mock(DlmsConnection.class);
        SessionLayer sessionLayer = mock(SessionLayer.class);

        final APduBlockingQueue blockingQueue = new APduBlockingQueue();

        doAnswer(new Answer<Void>() {
            @Override
            public Void answer(InvocationOnMock invocation) throws Throwable {
                byte[] arg0 = invocation.getArgumentAt(0, byte[].class);

                APdu data = APdu.decode(arg0);
                ACSE_apdu acseAPdu2 = data.acseAPdu;

                assertNotNull(acseAPdu2);
                RLRQ_apdu rlrq = acseAPdu2.rlrq;
                assertNotNull(rlrq);
                assertEquals(0L, rlrq.reason.value);

                ReleaseResponseReason responseReason = ReleaseResponseReason.NORMAL;
                Release_response_reason reason = new Release_response_reason(responseReason.getCode());
                Association_information userInformation = null;
                RLRE_apdu rlre = new RLRE_apdu(reason, userInformation);
                ACSE_apdu acseAPdu = new ACSE_apdu(null, null, null, rlre);
                APdu aPdu = new APdu(acseAPdu, null);
                blockingQueue.put(aPdu);
                return null;
            }
        }).when(sessionLayer).send(any(byte[].class), anyInt(), anyInt());

        Settings settings = mock(Settings.class);
        doReturn(100).when(settings).responseTimeout();

        setInternalState(connection, Settings.class, settings);
        setInternalState(connection, SessionLayer.class, sessionLayer);
        setInternalState(connection, APduBlockingQueue.class, blockingQueue);

        doCallRealMethod().when(connection).disconnect();
        connection.disconnect();
    }

    @Test()
    public void test2() throws Exception {
        // bytes from E350:
        // 0001 0001 0010 0005 6303800100
        byte[] releaseBytes = HexConverter.fromShortHexString("6303800100");

        APdu aPdu = APdu.decode(releaseBytes);

        ACSE_apdu acseAPdu = aPdu.acseAPdu;
        assertNotNull(acseAPdu);

        RLRE_apdu rlre = acseAPdu.rlre;
        assertNotNull(rlre);

        assertEquals(0L, rlre.reason.value);
    }
}
